/*******************************************************************************
 *	ChaseDa.c		CHASE_DA definititions
 *				T.Barnaby,	BEAM Ltd,	2007-01-16
 *******************************************************************************
 * Copyright (c) 2009 BEAM Ltd. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * This is a Linux driver and API library for the Chase Scientific Company's
 * CHASE_DA 8-Chan, 12-Bit, 150 MHz (MS/sec) Arbitrary Waveform Generator PCI board.
 */
#include <stdio.h>
#include <fcntl.h>
#include <stdint.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <chase_da.h>
#include <chase_da_lib.h>

#define	DEV_NAME	"/dev/chase_da-%d"
#define	CARD_NUM	4


typedef struct {
	int			fd;
	int			opened;
	int			boardType;
	int			numChannels;
	volatile uint16_t*	mem;
} Card;

static Card	cards[CARD_NUM + 1];

static inline void memWrite(Card* card, uint32_t wordAddress, uint16_t data){
//	printf("WriteMem: %x %x\n", wordAddress*2, data);
	card->mem[wordAddress] = data;
}

static inline uint16_t memRead(Card* card, uint32_t wordAddress){
//	printf("ReadMem: %x\n", wordAddress*2);
	return card->mem[wordAddress];
}

static int getCard(uint32_t cardNum, Card** card){
	if((cardNum < 1) || (cardNum > CARD_NUM))
		return CHASE_DA_ERROR_CARDNUM;

	if(cards[cardNum-1].opened != 1)
		return CHASE_DA_ERROR_CARDNOTOPEN;
	
	*card = &cards[cardNum-1];
	return 0;
}

int chase_da_CountCards(void){
	int	i;
	char	devName[32];
	int	fd;
	int	nc = 0;
	
	for(i = 0; i < CARD_NUM; i++){
		sprintf(devName, DEV_NAME, i);
		if(access(devName, O_RDWR))
			break;
		nc++;
	}
	return nc;
}

int chase_da_Open(uint32_t cardNum){
	int		fd;
	char		devName[32];
	ChaseDaInfo	info;
	
	if((cardNum < 1) || (cardNum > CARD_NUM))
		return CHASE_DA_ERROR_CARDNUM;
		
	sprintf(devName, DEV_NAME, cardNum-1);
	if((fd = open(devName, O_RDWR)) < 0){
		return CHASE_DA_ERROR_NOCARD;
	}
	
	cards[cardNum-1].fd = fd;

	// Map in cards memory
	if(ioctl(fd, CHASE_DA_CMD_GETINFO, &info) < 0){
		close(fd);
		fprintf(stderr, "chase_da_Open: Error ioctl: %s\n", strerror(errno));
		return CHASE_DA_ERROR_MISC;
	}

	if((cards[cardNum-1].mem = mmap(0, info.memLen, PROT_READ|PROT_WRITE, MAP_SHARED, fd, info.memPhysAddress)) == 0){
		close(fd);
		fprintf(stderr, "chase_da_Open: Error mmap: %s\n", strerror(errno));
		return CHASE_DA_ERROR_MISC;
	}
	cards[cardNum-1].opened = 1;
	cards[cardNum-1].boardType = info.boardType;

	// Check board type and set up parameters as required	
	if(info.boardType == CHASE_DA11000)
		cards[cardNum-1].numChannels = 1;
	else if(info.boardType == CHASE_DA8150)
		cards[cardNum-1].numChannels = 8;
	else
		return CHASE_DA_ERROR_MISC;

	chase_da_SetTriggerMode(cardNum, 0, 0);
	
	return 0;
}

int chase_da_Close(uint32_t cardNum){
	if((cardNum < 1) || (cardNum > CARD_NUM))
		return CHASE_DA_ERROR_CARDNUM;

	if(cards[cardNum-1].opened != 1)
		return CHASE_DA_ERROR_CARDNOTOPEN;
		
	close(cards[cardNum-1].fd);
	cards[cardNum-1].opened = 0;

	return 0;
}

int chase_da_BoardType(uint32_t cardNum){
	int	ret;
	Card*	card;
	
	if(ret = getCard(cardNum, &card))
		return ret;
	
	return card->boardType;
}

int chase_da_SetClock(uint32_t cardNum, uint32_t frequency){
	int	ret;
	Card*	card;
	
	if(ret = getCard(cardNum, &card))
		return ret;

	return ioctl(card->fd, CHASE_DA_CMD_SET_CLOCK, frequency);
}


int chase_da_SetTriggerMode(uint32_t cardNum, uint8_t mode, uint8_t extPol){
	int		ret;
	Card*		card;
	uint32_t	v;
	
	if(ret = getCard(cardNum, &card))
		return ret;

	return ioctl(card->fd, CHASE_DA_CMD_SET_TRIGGER, (mode << 8) | extPol);
}

int chase_da_SetSoftTrigger(uint32_t cardNum){
	int	ret;
	Card*	card;
	
	if(ret = getCard(cardNum, &card))
		return ret;

	return ioctl(card->fd, CHASE_DA_CMD_SOFT_TRIGGER, 0);
}

int chase_da_SetMarkers(uint32_t cardNum, uint32_t pointAddress, uint8_t nib1, uint8_t nib2){
	int	ret;
	Card*	card;
	
	pointAddress *= 2;

	if(ret = getCard(cardNum, &card))
		return ret;

	// Put PCI local bus to R/W mode
	ioctl(card->fd, CHASE_DA_CMD_MEM_ENABLE, 1);
	
	memWrite(card, ((pointAddress & 0xFFFFFFE0) | 0x06) / 2, memRead(card, ((pointAddress & 0xFFFFFFE0) | 0x06) / 2) | (nib1 << 12));
	memWrite(card, ((pointAddress & 0xFFFFFFE0) | 0x16) / 2, memRead(card, ((pointAddress & 0xFFFFFFE0) | 0x16) / 2) | (nib1 << 12));
	
	if(card->boardType == CHASE_DA11000){
		memWrite(card, ((pointAddress & 0xFFFFFFE0) | 0x04) / 2, memRead(card, ((pointAddress & 0xFFFFFFE0) | 0x04) / 2) | (nib2 << 12));
		memWrite(card, ((pointAddress & 0xFFFFFFE0) | 0x14) / 2, memRead(card, ((pointAddress & 0xFFFFFFE0) | 0x14) / 2) | (nib2 << 12));
	}

	// Put PCI local bus back to original state
	ioctl(card->fd, CHASE_DA_CMD_MEM_ENABLE, 0);

	return 0;
}

static void writeCommandWord(Card* card, uint32_t base, uint32_t offset, uint32_t value){
//	printf("writeCommandWord: %x,%x,%x\n", base + 16 + offset, base + 32 + offset, base + 48 + offset);
	memWrite(card, base + 16 + offset, (memRead(card, base + 16 + offset) & 0x0FFF) | (value << 12));
	memWrite(card, base + 32 + offset, (memRead(card, base + 32 + offset) & 0x0FFF) | (value << 12));
	memWrite(card, base + 48 + offset, (memRead(card, base + 48 + offset) & 0x0FFF) | (value << 12));
}

static void writeSegment(Card* card, uint32_t chanMask, ChaseDaSegment seg, uint32_t startAddress, uint32_t jmpAddress, int commandsOnly){
	int		i;
	int		c;
	uint32_t	pipeLinePre = 64;	// Leading Pad Size
	uint32_t	pipeLinePad = 64;	// Trailing Pad Size
	uint32_t	trigOffset = 0;
	uint32_t	dstAddr;
	uint32_t	cmd;
	uint32_t	begAddress = startAddress ;

	if(!commandsOnly){
		// Write start pad
		for(i = startAddress; i < startAddress + pipeLinePre; i++){
			memWrite(card, i, seg.beginPadValue);
		}

		if(card->boardType == CHASE_DA8150){
        		for(i = 0; i < seg.numPoints; i++){
				for(c = 0; c < 8; c++){
					if(c < 4)
						dstAddr = (8 * (i & ~1)) + 11 - c - (8 * (i & 1));
					else
						dstAddr = (8 * (i & ~1)) + 19 - c - (8 * (i & 1));
					if(chanMask & (1 << c)){
						memWrite(card, startAddress + pipeLinePre + dstAddr, seg.data[i]);
					}
					else {
						memWrite(card, startAddress + pipeLinePre + dstAddr, memRead(card, startAddress + pipeLinePre + dstAddr) & 0x0FFF);
					}
				}
			}
		}
		else if(card->boardType == CHASE_DA11000){
        		for(i = 0; i < seg.numPoints; i++){
				dstAddr = 14 + (i & 0xFFFFFFF0) + ~(i & 0x08) + ~((i & 0x04) >> 2) + ((i & 0x02) << 1) + ~((i & 0x01) << 1);
				memWrite(card, startAddress + pipeLinePre + dstAddr, seg.data[i]);
			}
		}
		else {
		}

		// Write end pad
		for(i = startAddress + pipeLinePre + seg.numPoints * card->numChannels; i < startAddress + pipeLinePre + seg.numPoints * card->numChannels + pipeLinePad; i++){
			memWrite(card, i, seg.endPadValue);
		}
	}
	else {
		// Clear all command codes
		for(i = 0; i < pipeLinePre + seg.numPoints * 8 + pipeLinePad; i++){
			memWrite(card, startAddress + i, memRead(card, startAddress + i) & 0x0FFF);
		}
	}

	// Write out commands
	if(seg.trigEnable){
		cmd = 3 << 2;
		trigOffset = 64;
	}
	else {
		cmd = 1 << 2;
		jmpAddress += pipeLinePre;
		begAddress += pipeLinePre;
	}
	
	// Write commands
	cmd |= (seg.numLoops >> 12) & 0x3;

	// Addresses are in 64byte increments
	jmpAddress >>= 6;
	begAddress >>= 6;

	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 0, cmd);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 1, seg.numLoops >> 8);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 4, seg.numLoops >> 4);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 5, seg.numLoops >> 0);

	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 10, jmpAddress >> 16);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 6, jmpAddress >> 12);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 7, jmpAddress >> 8);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 8, jmpAddress >> 4);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 9, jmpAddress >> 0);

	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 2, begAddress >> 16);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 12, begAddress >> 12);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 13, begAddress >> 8);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 14, begAddress >> 4);
	writeCommandWord(card, startAddress + seg.numPoints * card->numChannels + trigOffset, 15, begAddress >> 0);

	// Write out additional commands if triggered
	if(seg.trigEnable){
		cmd = 1 << 2;
		jmpAddress = (startAddress + pipeLinePre + seg.numPoints * card->numChannels) >> 6;
		begAddress = (pipeLinePre) >> 6;

		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 0, cmd);

		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 10, jmpAddress >> 16);
		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 6, jmpAddress >> 12);
		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 7, jmpAddress >> 8);
		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 8, jmpAddress >> 4);
		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 9, jmpAddress >> 0);

		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 2, begAddress >> 16);
		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 12, begAddress >> 12);
		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 13, begAddress >> 8);
		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 14, begAddress >> 4);
		writeCommandWord(card, startAddress + seg.numPoints * card->numChannels, 15, begAddress >> 0);
	}
}

int chase_da_CreateSegments(uint32_t cardNum, uint32_t chanMask, uint32_t numSegments, ChaseDaSegment* segmentsList){
	int		ret;
	Card*		card;
	int		i;
	uint32_t	startAddress = 0;
	uint32_t	startAddressList[32768];  // Store start addresses for each segment for jump reference
	
	if(ret = getCard(cardNum, &card))
		return ret;

	// Put PCI local bus to R/W mode
	ioctl(card->fd, CHASE_DA_CMD_MEM_ENABLE, 1);

	// Work out start addresses
	for(i = 0; i < numSegments; i++){
		startAddressList[i] = startAddress;
		startAddress = startAddress + 64 + (segmentsList[i].numPoints * card->numChannels) + 64;
	}

	// Write out Segments
	for(i = 0; i < numSegments; i++){
		writeSegment(card, chanMask, segmentsList[i], startAddressList[i], startAddressList[segmentsList[i].nextSegNum], 0);
	}

	// Put PCI local bus back to original state
	ioctl(card->fd, CHASE_DA_CMD_MEM_ENABLE, 0);

	return 0;
}

int chase_da_UpdateSegmentCmds(uint32_t cardNum, uint32_t chanMask, uint32_t numSegments, ChaseDaSegment* segmentsList){
	int		ret;
	Card*		card;
	int		i;
	uint32_t	startAddress = 0;
	uint32_t	startAddressList[32768];  // Store start addresses for each segment for jump reference
	
	if(ret = getCard(cardNum, &card))
		return ret;

	// Put PCI local bus to R/W mode
	ioctl(card->fd, CHASE_DA_CMD_MEM_ENABLE, 1);

	// Work out start addresses
	for(i = 0; i < numSegments; i++){
		startAddressList[i] = startAddress;
		startAddress = startAddress + 64 + (segmentsList[i].numPoints * card->numChannels) + 64;
	}

	// Write out Segments
	for(i = 0; i < numSegments; i++){
		writeSegment(card, chanMask, segmentsList[i], startAddressList[i], startAddressList[segmentsList[i].nextSegNum], 1);
	}

	// Put PCI local bus back to original state
	ioctl(card->fd, CHASE_DA_CMD_MEM_ENABLE, 0);

	return 0;
}

int chase_da_CreateSingleSegment(uint32_t cardNum, uint32_t chanNum, uint32_t numPoints, uint32_t numLoops, void* data, uint32_t trigEnable){
	ChaseDaSegment	seg;
	
	seg.segmentNum = 0;
	seg.data = data;
	seg.numPoints = numPoints;
	seg.numLoops = numLoops;
	seg.beginPadValue = 2047;
	seg.endPadValue = 2047;
	seg.trigEnable = trigEnable;
	seg.nextSegNum = 0;
	return chase_da_CreateSegments(cardNum, chanNum, 1, &seg);
}




int chase_da_CleanSingleSegmentArea(uint32_t cardNum, uint32_t numPointsPerChan){
	int			i;
	uint32_t		numPoints;
	int			ret;
	Card*			card;
	
	if(ret = getCard(cardNum, &card))
		return ret;

	// Put PCI local bus to R/W mode
	ioctl(card->fd, CHASE_DA_CMD_MEM_ENABLE, 1);

	numPoints = (numPointsPerChan * 8) & 0xFFFFFFC0;	// Make sure number is "MOD 4x16=64"  ==> AFTER TIMES 8 FOR 8-CHANNELS

	// CLEAN UPPER NIBBLE
	for(i = 0; i < (numPoints + 128); i++){
		card->mem[i] = 0;
	}

	// Put PCI local bus back to original state
	ioctl(card->fd, CHASE_DA_CMD_MEM_ENABLE, 0);

	return 0;
}

int chase_da_Set_Atten(uint32_t cardNum, uint32_t chanNum, uint32_t attenValue){
	int			ret;
	Card*			card;
	ChaseDaAttenuation	atten;
	
	if(ret = getCard(cardNum, &card))
		return ret;
		
	// Perform operation
	atten.chan = chanNum;
	atten.value = attenValue;
	if(ioctl(card->fd, CHASE_DA_CMD_SET_ATTENUATION, &atten) < 0){
		fprintf(stderr, "chase_da_Set_Atten: Error ioctl: %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

