/*******************************************************************************
 *	ChaseDa_lib.h		CHASE_DA definititions
 *				T.Barnaby,	BEAM Ltd,	2007-01-16
 *******************************************************************************
 * Copyright (c) 2009 BEAM Ltd. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * This is a Linux driver and API library for the Chase Scientific Company's
 * CHASE_DA 8-Chan, 12-Bit, 150 MHz (MS/sec) Arbitrary Waveform Generator PCI board.
 */
#ifndef _CHASE_DA_LIB_H
#define _CHASE_DA_LIB_H

#include <chase_da.h>

#define CHASE_DA_ERROR_NOCARD		1	///< Error no card is in the system
#define CHASE_DA_ERROR_CARDNUM		6	///< Card number is outside permitted range
#define CHASE_DA_ERROR_CARDNOTOPEN	5	///< Cannot open card
#define CHASE_DA_ERROR_MISC		-1	///< Misc error

#define CHASE_DA_MODE_OFF		0	///< Board disable
#define CHASE_DA_MODE_RUN		1	///< Board set to run
#define CHASE_DA_MODE_TRIGRUN		2	///< Board set to run on trigger

#define CHASE_DA_TRIG_POS		0	///< Positive edge trigger
#define CHASE_DA_TRIG_NEG		1	///< Negative edge trigger


typedef struct {
	uint32_t	segmentNum;	///< Current Segment Number
	uint16_t*	data;		///< Pointer to current user segment
					///< elements of one diminsional array must be of type uint16_t
	uint32_t	numPoints;	///< Number of points in segment (must be even multiple of 64)
	uint32_t	numLoops;	///< Number of times to repeat segment (applies to next segment)
	uint32_t	beginPadValue;	///< Pad value for beginning of triggered segment
	uint32_t	endPadValue;	///< Pad value for end of triggered segment
	uint32_t	trigEnable;	///< If > 0 then wait for trigger before going to next segment.
	uint32_t	nextSegNum;	///< Next segment to jump to after completion of current segment activities
} ChaseDaSegment;


int chase_da_CountCards(void);
int chase_da_Open(uint32_t cardNum);
int chase_da_Close(uint32_t cardNum);

int chase_da_BoardType(uint32_t cardNum);
int chase_da_SetClock(uint32_t cardNum, uint32_t frequency);

int chase_da_SetTriggerMode(uint32_t cardNum, uint8_t mode, uint8_t extPol);
int chase_da_SetSoftTrigger(uint32_t cardNum);
int chase_da_SetMarkers(uint32_t cardNum, uint32_t pointAddress, uint8_t nib1, uint8_t nib2);

int chase_da_CreateSingleSegment(uint32_t cardNum, uint32_t chanNum, uint32_t numPoints, uint32_t numLoops, void* dataPtr, uint32_t trigEnable);

int chase_da_CreateSegments(uint32_t cardNum, uint32_t chanNum, uint32_t numSegments, ChaseDaSegment* segmentsList);
int chase_da_UpdateSegmentCmds(uint32_t cardNum, uint32_t chanNum, uint32_t numSegments, ChaseDaSegment* segmentsList);

int chase_da_CleanSingleSegmentArea(uint32_t cardNum, uint32_t numPointsPerChan);
int chase_da_Set_Atten(uint32_t cardNum, uint32_t chanNum, uint32_t attenValue);

// Compatibility definitions
#define DA8150_ERROR_NOCARD		CHASE_DA_ERROR_NOCARD
#define DA8150_ERROR_CARDNUM		CHASE_DA_ERROR_CARDNUM
#define DA8150_ERROR_CARDNOTOPEN	CHASE_DA_ERROR_CARDNOTOPEN
#define DA8150_ERROR_MISC		CHASE_DA_ERROR_MISC
#define DA8150_MODE_OFF			CHASE_DA_MODE_OFF
#define DA8150_MODE_RUN			CHASE_DA_MODE_RUN
#define DA8150_MODE_TRIGRUN		CHASE_DA_MODE_TRIGRUN
#define DA8150_TRIG_POS			CHASE_DA_TRIG_POS
#define DA8150_TRIG_NEG			CHASE_DA_TRIG_NEG

#define	Da8150Segment			ChaseDaSegment

#define da8150_CountCards		chase_da_CountCards	
#define da8150_Open			chase_da_Open
#define da8150_Close			chase_da_Close
#define da8150_SetClock			chase_da_SetClock
#define da8150_SetTriggerMode		chase_da_SetTriggerMode
#define da8150_SetSoftTrigger		chase_da_SetSoftTrigger
#define da8150_SetMarkers		chase_da_SetMarkers
#define da8150_CreateSingleSegment	chase_da_CreateSingleSegment
#define da8150_CreateSegments		chase_da_CreateSegments
#define da8150_UpdateSegmentCmds	chase_da_UpdateSegmentCmds
#define da8150_CleanSingleSegmentArea	chase_da_CleanSingleSegmentArea
#define da8150_Set_Atten		chase_da_Set_Atten


#define DA11000_ERROR_NOCARD		CHASE_DA_ERROR_NOCARD
#define DA11000_ERROR_CARDNUM		CHASE_DA_ERROR_CARDNUM
#define DA11000_ERROR_CARDNOTOPEN	CHASE_DA_ERROR_CARDNOTOPEN
#define DA11000_ERROR_MISC		CHASE_DA_ERROR_MISC
#define DA11000_MODE_OFF		CHASE_DA_MODE_OFF
#define DA11000_MODE_RUN		CHASE_DA_MODE_RUN
#define DA11000_MODE_TRIGRUN		CHASE_DA_MODE_TRIGRUN
#define DA11000_TRIG_POS		CHASE_DA_TRIG_POS
#define DA11000_TRIG_NEG		CHASE_DA_TRIG_NEG

#define	Da11000Segment			ChaseDaSegment

#define da11000_CountCards		chase_da_CountCards	
#define da11000_Open			chase_da_Open
#define da11000_Close			chase_da_Close
#define da11000_SetClock		chase_da_SetClock
#define da11000_SetTriggerMode		chase_da_SetTriggerMode
#define da11000_SetSoftTrigger		chase_da_SetSoftTrigger
#define da11000_SetMarkers		chase_da_SetMarkers
#define da11000_CreateSingleSegment	chase_da_CreateSingleSegment
#define da11000_CreateSegments		chase_da_CreateSegments
#define da11000_UpdateSegmentCmds	chase_da_UpdateSegmentCmds
#define da11000_CleanSingleSegmentArea	chase_da_CleanSingleSegmentArea
#define da11000_Set_Atten		chase_da_Set_Atten

#endif


