/*******************************************************************************
 *	Da11000_lib.h		DA11000 definititions
 *				T.Barnaby,	BEAM Ltd,	2009-04-22
 *******************************************************************************
 * Copyright (c) 2009 BEAM Ltd. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * This is a Linux driver and API library for the Chase Scientific Company's
 * DA11000 8-Chan, 12-Bit, 150 MHz (MS/sec) Arbitrary Waveform Generator PCI board.
 */
#ifndef _DA11000_LIB_H
#define _DA11000_LIB_H

#include <stdint.h>

typedef uint32_t	DWORD;
typedef uint16_t	WORD;
typedef uint8_t		BYTE;
typedef void*		PVOID;
typedef void*		HANDLE;
typedef void*		P9030_HANDLE;
typedef P9030_HANDLE	P9030HANDLE;

#define DA11000_ERROR_NOCARD		1
#define DA11000_ERROR_CARDNUM		6
#define DA11000_ERROR_CARDNOTOPEN	5
#define DA11000_ERROR_MISC		-1

#define D8150_MODE_OFF		0		// Board is disabled
#define D8150_MODE_RUN		1		// Board is set to run
#define D8150_MODE_TRIGRUN	2		// Board is set to run on trigger

#define D8150_TRIG_POS		0		// Positive edge trigger
#define D8150_TRIG_NEG		1		// Negative edge trigger

typedef struct {
	uint32_t	segmentNum;	// Current Segment Number
	uint16_t*	data;		// Pointer to current user segment
					// elements of one diminsional array must be of type uint16_t
	uint32_t	numPoints;	// Number of points in segment (must be even multiple of 64)
	uint32_t	numLoops;	// Number of times to repeat segment (applies to next segment)
	uint32_t	beginPadValue;	// Pad value for beginning of triggered segment
	uint32_t	endPadValue;	// Pad value for end of triggered segment
	uint32_t	trigEnable;	// If > 0 then wait for trigger before going to next segment.
	uint32_t	nextSegNum;	// Next segment to jump to after completion of current segment activities
} Da8150Segment;

DWORD da11000_CountCards(void);
DWORD da11000_Open(DWORD CardNum);
DWORD da11000_Close(DWORD CardNum);

void da11000_SetClock(DWORD CardNum, DWORD Frequency);

void da11000_SetTriggerMode(DWORD CardNum, BYTE Mode, BYTE ExtPol);
void da11000_SetSoftTrigger(DWORD CardNum);
void da11000_SetMarkers(DWORD CardNum, DWORD PointAddr, BYTE Nib1, BYTE Nib2);
void da11000_SetOffset(DWORD CardNum, DWORD ChanNum, int Mode, int Offset);

void da11000_CleanSingleSegmentArea(DWORD CardNum, DWORD NumPointsPerChan);
void da11000_CreateSingleSegment(DWORD CardNum, DWORD ChanNum, DWORD NumPoints, DWORD NumLoops, PVOID UserArrayPtr, DWORD TrigEn);

void da11000_CreateSegments(DWORD CardNum, DWORD ChanNum, DWORD NumSegments, PVOID PtrToSegmentsList);
void da11000_UpdateSegmentCmds(DWORD CardNum, DWORD ChanNum, DWORD NumSegments, PVOID PtrToSegmentsList);

void da11000_Set_Atten(DWORD CardNum, DWORD ChanNum, DWORD Atten_Value);


/*******************************************************************************
 *	P9030 interface code
 *******************************************************************************
 */
#define P9030_OPEN_FIX_BIT7	0

// PLX register definitions 
enum {
    P9030_LAS0RR      = 0x00,
    P9030_LAS1RR      = 0x04,
    P9030_LAS2RR      = 0x08,
    P9030_LAS3RR      = 0x0c,
    P9030_EROMRR      = 0x10,
    P9030_LAS0BA      = 0x14,
    P9030_LAS1BA      = 0x18,
    P9030_LAS2BA      = 0x1c,
    P9030_LAS3BA      = 0x20,
    P9030_EROMBA      = 0x24,
    P9030_LAS0BRD     = 0x28,
    P9030_LAS1BRD     = 0x2c,
    P9030_LAS2BRD     = 0x30,
    P9030_LAS3BRD     = 0x34,
    P9030_EROMBRD     = 0x38,
    P9030_CS0BASE     = 0x3c,
    P9030_CS1BASE     = 0x40,
    P9030_CS2BASE     = 0x44,
    P9030_CS3BASE     = 0x48,
    P9030_INTCSR      = 0x4c,
    P9030_PROT_AREA   = 0x4e,
    P9030_CNTRL       = 0x50,
    P9030_GPIOC       = 0x54,
    P9030_PMDATASEL   = 0x70,
    P9030_PMDATASCALE = 0x74,
};

typedef enum
{
    P9030_ADDR_REG     = 0,
    P9030_ADDR_REG_IO  = 1,
    P9030_ADDR_SPACE0  = 2,
    P9030_ADDR_SPACE1  = 3,
    P9030_ADDR_SPACE2  = 4,
    P9030_ADDR_SPACE3  = 5,
    P9030_ADDR_EPROM   = 6,
} P9030_ADDR;

typedef enum
{
    P9030_MODE_BYTE   = 0,
    P9030_MODE_WORD   = 1,
    P9030_MODE_DWORD  = 2
} P9030_MODE;


int P9030_CountCards(void);
int P9030_Open (P9030_HANDLE *phPlx, DWORD dwVendorID, DWORD dwDeviceID, DWORD nCardNum);
int P9030_WriteSpaceWord (P9030_HANDLE hPlx, P9030_ADDR addrSpace, DWORD dwOffset, WORD data);
WORD P9030_ReadSpaceWord (P9030_HANDLE hPlx, P9030_ADDR addrSpace, DWORD dwOffset);
DWORD P9030_ReadReg (P9030_HANDLE hPlx, DWORD dwReg);
void P9030_Close (P9030_HANDLE hPlx);

#endif


