/*******************************************************************************
 *	Da8150.c		DA8150 definititions
 *				T.Barnaby,	BEAM Ltd,	2007-01-16
 *******************************************************************************
 * Copyright (c) 2007 BEAM Ltd. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * This is a Linux driver and API library for the Chase Scientific Company's
 * DA8150 8-Chan, 12-Bit, 150 MHz (MS/sec) Arbitrary Waveform Generator PCI board.
 */
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <chase_da.h>
#include <da8150_lib.h>

#define	DEV_NAME	"/dev/chase_da-%d"
#define	CARD_NUM	4

#define DA8150_ERROR_NOCARD		1
#define DA8150_ERROR_CARDNUM		6
#define DA8150_ERROR_CARDNOTOPEN	5
#define DA8150_ERROR_MISC		-1

typedef struct {
	int			fd;
	int			opened;
	volatile uint16_t*	reg;
	size_t			regLen;
	volatile uint16_t*	mem;
	size_t			memLen;
} Card;

static Card	cards[CARD_NUM + 1];

static int getCard(uint32_t cardNum, Card** card){
	if((cardNum < 1) || (cardNum > CARD_NUM))
		return DA8150_ERROR_CARDNUM;

	if(cards[cardNum-1].opened != 1)
		return DA8150_ERROR_CARDNOTOPEN;
	
	*card = &cards[cardNum-1];
	return 0;
}

int P9050_CountCards(void){
	int	i;
	char	devName[32];
	int	nc = 0;
	
	for(i = 0; i < CARD_NUM; i++){
		sprintf(devName, DEV_NAME, i);
		if(access(devName, O_RDWR))
			break;
		nc++;
	}
	return nc;
}

int P9050_Open(P9050_HANDLE* phPlx, DWORD dwVendorID, DWORD dwDeviceID, DWORD cardNum, DWORD options){
	int		fd;
	char		devName[32];
	ChaseDaInfo	info;
	Card*		card = 0;
	
//	printf("P9050_Open: %d\n", cardNum);
	if((cardNum < 0) || (cardNum >= CARD_NUM)){
		fprintf(stderr, "P9050_Open: Card Number error: %d\n", cardNum);
		return 0;
	}
		
	sprintf(devName, DEV_NAME, cardNum);
	if((fd = open(devName, O_RDWR)) < 0){
		fprintf(stderr, "P9050_Open: Unable to open Card Number: %d %s\n", cardNum, strerror(errno));
		return 0;
	}
	
	card = &cards[cardNum];
	
	card->fd = fd;

	// Map in cards memory
	if(ioctl(card->fd, CHASE_DA_CMD_GETINFO, &info) < 0){
		close(card->fd);
		fprintf(stderr, "P9050_Open: Error ioctl: %s\n", strerror(errno));
		return 0;
	}

	if((card->reg = (uint16_t*)mmap(0, info.regsLen, PROT_READ|PROT_WRITE, MAP_SHARED, fd, info.regsPhysAddress)) == 0){
		close(card->fd);
		fprintf(stderr, "P9050_Open: Error mmap: %s\n", strerror(errno));
		return 0;
	}
	if((card->mem = (uint16_t*)mmap(0, info.memLen, PROT_READ|PROT_WRITE, MAP_SHARED, fd, info.memPhysAddress)) == 0){
		close(card->fd);
		fprintf(stderr, "P9050_Open: Error mmap: %s\n", strerror(errno));
		return 0;
	}

	card->regLen = info.regsLen;
	card->memLen = info.memLen;
	card->opened = 1;
	*phPlx = card;

//	printf("P9050_Open: fd: %d\n", card->fd);
	return 1;
}

int P9050_WriteSpaceWord (P9050_HANDLE hPlx, P9050_ADDR addrSpace, DWORD dwOffset, WORD data){
	volatile uint16_t*	reg;
	volatile uint16_t*	mem;
	Card*			card = (Card*)hPlx;

	reg = card->reg;
	mem = card->mem;

//	printf("P9050_WriteSpaceWord: %d %x %x\n", addrSpace, dwOffset, data);
	if(addrSpace == P9050_ADDR_SPACE0){
		mem[dwOffset/2] = data;
	}
	else if(addrSpace == P9050_ADDR_SPACE1){
		reg[dwOffset/2] = data;
	}
	else {
		fprintf(stderr, "P9050_WriteSpaceWord: Error: Write to incorrect space: %d\n", addrSpace);
	}

	return 0;
}

WORD P9050_ReadSpaceWord (P9050_HANDLE hPlx, P9050_ADDR addrSpace, DWORD dwOffset){
	volatile uint16_t*	reg;
	volatile uint16_t*	mem;
	Card*			card = (Card*)hPlx;

	reg = card->reg;
	mem = card->mem;

//	printf("P9050_ReadSpaceWord: %d %x\n", addrSpace, dwOffset);
	if(addrSpace == P9050_ADDR_SPACE0){
		return mem[dwOffset/2];
	}
	else if(addrSpace == P9050_ADDR_SPACE1){
		return reg[dwOffset/2];
	}
	else {
		fprintf(stderr, "P9050_ReadSpaceWord: Error: Read from incorrect space: %d\n", addrSpace);
	}
	return 0;
}

void P9050_Close (P9050_HANDLE hPlx){
	Card*	card = (Card*)hPlx;

//	printf("P9050_Close: fd: %d\n", card->fd);
	munmap((void*)card->mem, card->memLen);
	munmap((void*)card->reg, card->regLen);
 	close(card->fd);
	card->opened = 0;
}
